package bodycheck

import (
	"bytes"
	"encoding/json"
	"errors"
	"io/ioutil"
	"net/http"
	"strings"

	"github.com/astaxie/beego"
	"github.com/astaxie/beego/config"
)

// WechatInfo 微信接入
func (c *BodyCheckController) WechatInfo() {
	// signature := c.GetString("signature")
	// timestamp := c.GetString("timestamp")
	// nonce := c.GetString("nonce")
	echostr := c.GetString("echostr")
	c.ResponseRaw([]byte(echostr))
}

// GetQrcodeURL 获取二维码信息
func (c *BodyCheckController) GetQrcodeURL() {
	EquipmentID := c.GetString("EquipmentId")
	qrcodePrefix := beego.AppConfig.String("qrcodePrefix")
	qrcode, err := c.weixin.GetTempStrQRCode(qrcodePrefix + EquipmentID)
	if err != nil {
		c.ResponseJson(map[string]interface{}{
			"Status":  false,
			"Message": err.Error(),
		})
	}
	c.ResponseJson(map[string]interface{}{
		"Status":    true,
		"Message":   "",
		"QrcodeUrl": qrcode,
	})
}

// WxReceive 微信消息接收
func (c *BodyCheckController) WxReceive() {
	r := c.Ctx.Request
	defer r.Body.Close()
	con, _ := ioutil.ReadAll(r.Body) //获取post的数据
	val, err := c.weixin.TransformMessage(string(con))
	if err != nil {
		beego.Error(err.Error())
		c.ResponseRaw([]byte(""))
	}

	if val["MsgType"] == "event" {
		qrcodePrefix := beego.AppConfig.String("qrcodePrefix")
		if (val["Event"] == "subscribe" && strings.Contains(val["EventKey"], "qrscene_"+qrcodePrefix)) || val["Event"] == "SCAN" {
			EquipmentID := strings.Split(val["EventKey"], qrcodePrefix)[1]
			messagetxt, err := c.scanDo(val, EquipmentID)
			if err != nil {
				beego.Error(err.Error())
				c.ResponseRaw([]byte(""))
			}
			c.ResponseRaw(messagetxt)
		}
	}
	c.ResponseRaw([]byte(""))
}

func (c *BodyCheckController) scanDo(val map[string]string, EquipmentID string) ([]byte, error) {
	if val["FromUserName"] == "" {
		return nil, errors.New("openid为空")
	}

	user, err := c.weixin.GetUserDetail(val["FromUserName"])
	if err != nil {
		return nil, err
	}

	// 判断用户是否存在
	getUser, err := c.dao.GetUserByOpenID(user["openid"].(string))
	if err != nil {
		return nil, err
	}
	name := user["nickname"].(string)
	name = UnicodeEmojiCode(name)
	user["nickname"] = name
	if getUser == nil || getUser.Id < 1 {
		// 新增用户
		var newUser = model.CustomerUser{}
		newUser.Username = name
		newUser.WcNickname = name
		sex := int(user["sex"].(float64))
		if sex == 2 {
			sex = 1
		} else {
			sex = 0
		}
		newUser.Sex = sex
		newUser.OpenId = user["openid"].(string)
		newUser.Icon = user["headimgurl"].(string)
		newUser.UserType = 4
		newUser.ActualBalance = "0.00"
		newUser.GiftBalance = "0.00"
		_, err = c.dao.SaveUser(newUser)
		if err != nil {
			return nil, err
		}
	}

	type WechatInfo struct {
		EquipmentId    string
		WXID           int
		WechatUserInfo map[string]interface{}
	}
	wxconf, _ := config.NewConfig("ini", "conf/wechat.conf")
	user["wechataccount"] = user["openid"].(string)
	user["wechatofficial"] = wxconf.String("appid")
	var userinfo = WechatInfo{}
	userinfo.EquipmentId = EquipmentID
	userinfo.WechatUserInfo = user
	userinfo.WXID = 1

	apiPrefix := beego.AppConfig.String("postwechatInfo")

	b, err := json.Marshal(userinfo)
	if err != nil {
		return nil, err
	}

	body := bytes.NewBuffer([]byte(b))
	beego.Error(body)
	res, err := http.Post(apiPrefix, "application/json;charset=utf-8", body)
	if err != nil {
		beego.Error(err)
		return nil, err
	}
	result, err := ioutil.ReadAll(res.Body)
	res.Body.Close()
	if err != nil {
		beego.Error(err)
		return nil, err
	}
	var postResult struct {
		Status  bool
		Message string
	}
	if err = json.Unmarshal(result, &postResult); err != nil {
		beego.Error(err)
		return nil, err
	}

	if !postResult.Status {
		beego.Error(postResult.Message)
		return nil, errors.New(postResult.Message)
	}

	messagetxt, err := c.weixin.ResponseMessageText(user["openid"].(string), beego.AppConfig.String("followText"))
	if err != nil {
		beego.Error(err)
		return nil, err
	}
	return messagetxt, nil
}

// UnicodeEmojiCode 过滤表情
func UnicodeEmojiCode(s string) string {
	ret := ""
	rs := []rune(s)
	for i := 0; i < len(rs); i++ {
		if len(string(rs[i])) == 4 {
			// u := `[\u` + strconv.FormatInt(int64(rs[i]), 16) + `]`
			ret += ""
		} else {
			ret += string(rs[i])
		}
	}
	return ret
}
